# -*- coding: utf-8 -*-
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.support.ui import Select
from selenium.common.exceptions import StaleElementReferenceException, NoSuchElementException, WebDriverException
import unittest
import os
import sys
import time
import codecs
import re
import socket


def urlJoin(url_list):
	return '/'.join(s.strip('/') for s in url_list)

def urlFromHostname(hostname, port=80):
	try:
		return 'http://' + socket.gethostbyname(hostname) + ':' + str(port)
	except socket.gaierror as e:
		return 'unknown'

class QfqSeleniumTestCase(unittest.TestCase):
    """initialize selenium and add custom commands and asserts"""
    max_number_of_log_files_to_keep = 100

    # These variables can be overwritten by environment variables of the same name
    CHROMEDRIVER_PATH = "chromedriver"
    SELENIUM_LOGS_PATH = os.getcwd()
    SELENIUM_HEADLESS = 'yes'   # set environment variable to 'no' to turn off
    SELENIUM_URL = urlFromHostname('typo3container')

    @classmethod
    def setUpClass(cls):
        """executed by unittest once before any tests are executed"""

        # read environment variables
        cls.CHROMEDRIVER_PATH = os.environ.get('CHROMEDRIVER_PATH', cls.CHROMEDRIVER_PATH)
        cls.SELENIUM_LOGS_PATH = os.environ.get('SELENIUM_LOGS_PATH', cls.SELENIUM_LOGS_PATH)
        cls.SELENIUM_HEADLESS = os.environ.get('SELENIUM_HEADLESS', cls.SELENIUM_HEADLESS)
        cls.SELENIUM_URL = os.environ.get('SELENIUM_URL', cls.SELENIUM_URL)

        # setup log directory, delete very old log files
        cls.selenium_logs_dir = 'selenium_logs'
        cls.selenium_logs_dir_path = os.path.join(cls.SELENIUM_LOGS_PATH, cls.selenium_logs_dir)
        if not os.path.exists(cls.selenium_logs_dir_path):
            os.makedirs(cls.selenium_logs_dir_path)
        existing_log_files = os.listdir(cls.selenium_logs_dir_path)
        existing_log_files.sort()
        log_files_to_delete = existing_log_files[:-cls.max_number_of_log_files_to_keep]
        log_file_pattern = re.compile('[0-9]{8}-{1}[0-9]{6}.*')
        for filename in log_files_to_delete:
            if not log_file_pattern.match(filename):
                print 'non log file in log directory found: ', filename
                continue
            file_path = os.path.join(cls.selenium_logs_dir_path, filename)
            os.remove(file_path)

        # initialize webdriver
        chrome_options = Options()
        if  cls.SELENIUM_HEADLESS != 'no':
            chrome_options.add_argument("--headless")
        chrome_options.add_argument("--window-size=1920,1080")
        chrome_options.add_argument("--disable-dev-shm-usage")
        chrome_options.add_argument("--no-sandbox")
        desired_capabilities = {'UNEXPECTED_ALERT_BEHAVIOUR': 'ignore'}
        cls.driver = webdriver.Chrome(chrome_options=chrome_options, 
            executable_path=cls.CHROMEDRIVER_PATH, 
            desired_capabilities=desired_capabilities)
        cls.driver.implicitly_wait(30)

    @classmethod
    def tearDownClass(cls):
        """executed by unittest after all test cases were executed"""
        cls.driver.quit()
        
    def tearDown(self):
        """executed by unittest after every single test case"""

        # save website state on failure
        if sys.exc_info()[0]:
            filename = time.strftime("%Y%m%d-%H%M%S") + '_' + self._testMethodName
            screenshot_file_path = self.qfq_save_screenshot(filename)
            html_file_path = self.qfq_save_html(filename)
            print 'Test failed.'
            print 'Webpage screenshot saved to ' + screenshot_file_path
            print 'Webpage Html saved to ' + html_file_path
            print '!!! ATTENTION !!!: If you get the error "unexpected alert open",' \
                + 'there must be another error above which actually triggers the test failure.'

    def _prepare_log_file_path(self, filename, suffix=''):
        file_path = os.path.join(self.selenium_logs_dir_path, filename + suffix)
        return file_path

    def _retry_on_certain_exceptions(self, function, retries=10):
        while retries >= 0:
            try:
                function()
                break
            except (StaleElementReferenceException, NoSuchElementException, WebDriverException):
                retries -= 1

    def qfq_save_screenshot(self, filename):
        screenshot_file_path = self._prepare_log_file_path(filename, '.png')
        self.driver.save_screenshot(screenshot_file_path)
        return screenshot_file_path

    def qfq_save_html(self, filename):
        html_file_path = self._prepare_log_file_path(filename, '.html')
        with codecs.open(html_file_path, "w", "utf-8") as f:
            f.write(self.driver.page_source)
        return html_file_path

    def qfq_goto(self, web_path):
    	url = urlJoin([self.SELENIUM_URL, web_path])
    	self.driver.get(url)

    def qfq_wait(self, seconds):
        time.sleep(seconds)

    def qfq_assert_text_exists(self, text):
        self.assertTrue(text in self.driver.page_source)

    def qfq_assert_text_absent(self, text):
        self.assertTrue(text not in self.driver.page_source)

    def qfq_find_by_data_ref(self, data_reference):
        return self.driver.find_element_by_css_selector("[data-reference='{}']".format(data_reference))

    def qfq_fill_textfield(self, data_reference, text):
        self.qfq_find_by_data_ref(data_reference).send_keys(text)

    def qfq_dropdown_select(self, data_reference, option):
        Select(self.qfq_find_by_data_ref(data_reference)).select_by_visible_text(option)

    def qfq_click_new_form_button(self):
        self.qfq_find_by_data_ref('newForm').click()

    def qfq_click_close_form_button(self):
        def f(): self.driver.find_element_by_id('close-button').click()
        self._retry_on_certain_exceptions(f)

    def qfq_click_save_form_button(self):
        self.driver.find_element_by_id('save-button').click()

    def qfq_click_button_with_text(self, text):
        def f(): self.driver.find_element_by_xpath("//button[text()='{}']".format(text)).click()
        self._retry_on_certain_exceptions(f)